'''
Created on 7 Dec 2015

@author: Ronny Andersson (ronny@andersson.tk)
@copyright: (c) 2015 Ronny Andersson
@license: MIT
'''

# Standard library
import logging

# Internal
import zignal


def fake_system(x, fs=None):
    f1 = zignal.filters.biquads.RBJ(filtertype="peak", gaindb=-30, f0=10,   Q=0.707*10, fs=fs)
    f2 = zignal.filters.biquads.RBJ(filtertype="peak", gaindb=50,  f0=100,  Q=0.707*10, fs=fs)
    f3 = zignal.filters.biquads.RBJ(filtertype="peak", gaindb=-60, f0=1000, Q=0.707*10, fs=fs)

    y = zignal.Audio(fs=fs, initialdata=x)
    y.samples = f1.filter_samples(y.samples)
    y.samples = f2.filter_samples(y.samples)
    y.samples = f3.filter_samples(y.samples)

    y.delay(24000)

    return y


if __name__ == '__main__':
    logging.basicConfig(
        format="%(levelname)-8s: %(module)s.%(funcName)-15s %(message)s",
        level="DEBUG",
        )
    # some libraries are noisy in DEBUG
    logging.getLogger("matplotlib").setLevel(logging.INFO)
    logging.getLogger("PIL").setLevel(logging.WARNING)

    # sample rate
    fs = 48000

    # mls order. Relates to the sample rate and the length of the impulse we want to capture
    N = 16

    # The number of repeated mls sequences. More repetitions gives a better S/N ratio
    rep = 5

    # An emphasis filter. Here we put more energy into the low frequencies to stimulate
    # the systems response in the lower frequency area, giving a better frequency response
    f = zignal.filters.biquads.RBJ(filtertype="highshelf", gaindb=-10, f0=100, Q=0.707, fs=fs)
    B, A = f.get_coefficients()

    # Get a set of taps related to the mls order N. Either use the same all the time
    # or select a random set at each run.
    taps = zignal.measure.mls.TAPS[N][0]
    #taps = zignal.measure.mls.get_random_taps(N)

    # Create the mls instance and its full sequence.
    mls = zignal.measure.mls.MLS(N=N, taps=taps, fs=fs, repeats=rep, B=B, A=A)

    print(repr(mls))
    print(mls)

    mls.plot(label="mls signal unfiltered")

    # apply the emphasis
    mls.apply_emphasis()

    mls.plot(label="mls signal post emphasis")

    # send the signal through a system
    y = fake_system(mls.samples, fs=fs)
    y.plot(label="signal post system")

    # apply deemphasis before we extract the impulse response. Flip comments
    # to "forget" to apply the deemphasis
    z = mls.apply_deemphasis(y.samples)
    #z = y.samples

    # extract the impulse response. Will throw away the first sequence and average the others
    k = mls.get_impulse(z)
    k.plot(label="impulse in time domain")
    k.plot_fft(window='rectangular', normalise=False)

    # Note how the lowest filter in the system (the one at 10 Hz) is barely detected. We
    # can also see that the impulse in the time domain hasn't finished ringing during the
    # time. This is all related. Increasing the MLS order N will capture a longer impulse
    # and give better performace at lower frequencies. This is always a tradeoff between
    # performance and resolution.
    #
    # The impulse also wraps around when plotted in the time domain. This is not a
    # problem when we are interested in the magnitude response. If we are interested in
    # the phase response the delay between time=0 and the "start" of the impulse will
    # determine the amount of phase unwrapping we will have to apply.



"""
This is the zignal library

@author: Ronny Andersson (ronny@andersson.tk)
@copyright: (c) 2013 Ronny Andersson
@license: MIT
"""

__version__ = "0.7.0"

# must be imported first
from .audio import *    # isort:skip                        # noqa: F403

# Local folders
from . import filters, measure, music, sndcard

__all__ = [
    'filters',
    'measure',
    'music',
    'sndcard',
    ]
__all__.extend(audio.__all__)       #@UndefinedVariable    # noqa: F405,E262



'''
Created on 28 Feb 2014

@author: Ronny Andersson (ronny@andersson.tk)
@copyright: (c) 2014 Ronny Andersson
@license: MIT
'''

# Standard library
import logging
import unittest

# Third party
import numpy as np

# Internal
from zignal import Audio


class Test_ConvertBackToBack(unittest.TestCase):
    def setUp(self):
        self.x = Audio(fs=10, initialdata=np.zeros((10, 1)))
        self.x.samples[0] = -1.0
        self.x.samples[1] =  1.0                                            # noqa: E222
        #print(self.x)
        #print(self.x.samples)
        #print()

    def quantization_step_size(self, bits):
        return 2**-(bits-1)

    def test_float_to_int8_to_float64(self):
        self.x.convert_to_integer(targetbits=8)
        self.x.convert_to_float(targetbits=64)
        #print(self.x)
        #print(self.x.samples)

        q = self.quantization_step_size(8)
        self.assertAlmostEqual(self.x.samples[0], -1.0 + q, places=20)
        self.assertAlmostEqual(self.x.samples[1],  1.0 - q, places=20)

    def test_float_to_int16_to_float64(self):
        self.x.convert_to_integer(targetbits=16)
        self.x.convert_to_float(targetbits=64)
        #print(self.x)
        #print(self.x.samples)

        q = self.quantization_step_size(16)
        self.assertAlmostEqual(self.x.samples[0], -1.0 + q, places=20)
        self.assertAlmostEqual(self.x.samples[1],  1.0 - q, places=20)

    def test_float_to_int32_to_float64(self):
        self.x.convert_to_integer(targetbits=32)
        self.x.convert_to_float(targetbits=64)
        #print(self.x)
        #print(self.x.samples)

        q = self.quantization_step_size(32)
        self.assertAlmostEqual(self.x.samples[0], -1.0 + q, places=20)
        self.assertAlmostEqual(self.x.samples[1],  1.0 - q, places=20)


class Test_ConvertFloatToInt(unittest.TestCase):
    def setUp(self):
        self.x = Audio(fs=10, initialdata=np.zeros((10, 1)))
        self.x.samples[0] = -1.0
        self.x.samples[1] =  1.0                                            # noqa: E222
        #print(self.x)
        #print(self.x.samples)
        #print()

    def convert(self, targetbits=None):
        self.x.convert_to_integer(targetbits=targetbits)
        #print(self.x)
        #print(self.x.samples)

        self.assertIsInstance(self.x.samples, np.ndarray)

    def test_int8(self):
        self.convert(targetbits=8)
        self.assertTrue(self.x.samples.dtype == np.int8)

        # 8 bits 2's complement
        # min is -128
        # max is  127
        self.assertEqual(self.x.samples[0], -127)   # must be symmetrical
        self.assertEqual(self.x.samples[1],  127)   # must be symmetrical

    def test_int16(self):
        self.convert(targetbits=16)
        self.assertTrue(self.x.samples.dtype == np.int16)

        # 16 bits 2's complement
        # min is -32768
        # max is  32767
        self.assertEqual(self.x.samples[0], -32767)     # must be symmetrical
        self.assertEqual(self.x.samples[1],  32767)     # must be symmetrical

    def test_int32(self):
        self.convert(targetbits=32)
        self.assertTrue(self.x.samples.dtype == np.int32)

        # 32 bits 2's complement
        # min is -2147483648
        # max is  2147483647
        self.assertEqual(self.x.samples[0], -2147483647)    # must be symmetrical
        self.assertEqual(self.x.samples[1],  2147483647)    # must be symmetrical


class Test_ConvertIntToFloat32(unittest.TestCase):
    def convert(self, y, targetbits=None):
        #print(y)
        #print(y.pretty_string_samples())
        #print()

        y.convert_to_float(targetbits=targetbits)
        #print(y)
        #print(y.pretty_string_samples())
        #print()

    def test_int8(self):
        x = Audio(fs=10, initialdata=np.zeros((10, 1), dtype=np.int8))
        x.samples[0] = -128
        x.samples[1] =  127                                                 # noqa: E222

        self.convert(x, targetbits=32)

        self.assertAlmostEqual(x.samples[0], -1.0,      places=20)
        self.assertAlmostEqual(x.samples[1], 127/128,   places=20)

    def test_int16(self):
        x = Audio(fs=10, initialdata=np.zeros((10, 1), dtype=np.int16))
        x.samples[0] = -32768
        x.samples[1] =  32767                                               # noqa: E222

        self.convert(x, targetbits=32)

        self.assertAlmostEqual(x.samples[0], -1.0,          places=20)
        self.assertAlmostEqual(x.samples[1], 32767/32768,   places=20)

    def test_int32(self):
        x = Audio(fs=10, initialdata=np.zeros((10, 1), dtype=np.int32))
        x.samples[0] = -2147483648
        x.samples[1] =  2147483647                                          # noqa: E222

        self.convert(x, targetbits=32)

        # resolution is lost in this conversion, however we should be close enough.
        self.assertAlmostEqual(x.samples[0], -1.0, places=20)   # exact value
        self.assertAlmostEqual(x.samples[1],  1.0, places=20)   # close enough


if __name__ == "__main__":
    logging.basicConfig(
        format="%(levelname)-8s: %(module)s.%(funcName)-15s %(message)s",
        level="DEBUG",
        )
    # some libraries are noisy in DEBUG
    logging.getLogger("matplotlib").setLevel(logging.INFO)
    logging.getLogger("PIL").setLevel(logging.WARNING)

    # from command line:
    # $ python -m unittest -v zignal/tests/<filename>.py
    unittest.main(verbosity=2)



'''
Created on 23 Oct 2014

@author: Ronny Andersson (ronny@andersson.tk)
@copyright: (c) 2014 Ronny Andersson
@license: MIT
'''

# Standard library
import logging
import unittest

# Third party
import numpy as np

# Internal
from zignal import db2lin, db2pow, lin2db, pow2db


class Test_back_to_back(unittest.TestCase):
    def test_nested(self):
        x = lin2db(db2pow(pow2db(db2lin(1.234567))))
        self.assertAlmostEqual(x,       1.234567,  places=6)

    def test_lin_to_db_to_lin(self):
        x = lin2db(db2lin(              1.234567))                              # noqa: E201
        self.assertAlmostEqual(x,       1.234567,  places=6)

    def test_pow_to_db_to_pow(self):
        x = pow2db(db2pow(              1.234567))                              # noqa: E201
        self.assertAlmostEqual(x,       1.234567,  places=6)

    def test_lin_to_db_to_lin_arrays(self):
        x = lin2db(db2lin((             1.234567,   2.345678)))                 # noqa: E201
        self.assertEqual(x.ndim, 1)
        self.assertEqual(len(x), 2)
        self.assertAlmostEqual(x[0],    1.234567,               places=6)
        self.assertAlmostEqual(x[1],                2.345678,   places=6)

    def test_pow_to_db_to_pow_arrays(self):
        x = pow2db(db2pow((             1.234567,   2.345678)))                 # noqa: E201
        self.assertEqual(x.ndim, 1)
        self.assertEqual(len(x), 2)
        self.assertAlmostEqual(x[0],    1.234567,               places=6)
        self.assertAlmostEqual(x[1],                2.345678,   places=6)


class Test_lin_to_db_known_values(unittest.TestCase):
    def test_known_value_1(self):
        x = lin2db(1)
        self.assertAlmostEqual(x,  0.0,  places=6)

    def test_known_value_0_1(self):
        x = lin2db(0.1)
        self.assertAlmostEqual(x, -20.0, places=6)

    def test_known_value_0_5(self):
        x = lin2db(0.5)
        self.assertAlmostEqual(x, -6.02, places=2)

    def test_known_value_2(self):
        x = lin2db(2)
        self.assertAlmostEqual(x,  6.02, places=2)

    def test_known_value_0_0(self):
        x = lin2db(0.0)
        self.assertTrue(np.isneginf(x))


class Test_db_to_lin_known_values(unittest.TestCase):
    def test_known_value_0(self):
        x = db2lin(0)
        self.assertAlmostEqual(x, 1.0, places=6)

    def test_known_value_neg20(self):
        x = db2lin(-20)
        self.assertAlmostEqual(x, 0.1, places=6)

    def test_known_value_neg6(self):
        x = db2lin(-6)  # -6.020599913
        self.assertAlmostEqual(x, 0.5, places=2)

    def test_known_value_6(self):
        x = db2lin(6)
        self.assertAlmostEqual(x, 2.0, places=2)

    def test_known_value_neg_inf(self):
        x = db2lin(float('-inf'))
        self.assertAlmostEqual(x,  0.0, places=6)


class Test_pow_to_db_known_values(unittest.TestCase):
    def test_known_value_1(self):
        x = pow2db(1)
        self.assertAlmostEqual(x,  0.0,  places=6)

    def test_known_value_0_1(self):
        x = pow2db(0.1)
        self.assertAlmostEqual(x, -10.0, places=6)

    def test_known_value_0_5(self):
        x = pow2db(0.5)
        self.assertAlmostEqual(x, -3.01, places=2)

    def test_known_value_2(self):
        x = pow2db(2)
        self.assertAlmostEqual(x,  3.01, places=2)

    def test_known_value_0_0(self):
        x = pow2db(0.0)
        self.assertTrue(np.isneginf(x))


class Test_db_to_pow_known_values(unittest.TestCase):
    def test_known_value_0(self):
        x = db2pow(0)
        self.assertAlmostEqual(x, 1.0, places=6)

    def test_known_value_3(self):
        x = db2pow(3)
        self.assertAlmostEqual(x, 2.0, places=2)

    def test_known_value_6(self):
        x = db2pow(6.02)
        self.assertAlmostEqual(x, 4.0, places=2)

    def test_known_value_neg_3(self):
        x = db2pow(-3)
        self.assertAlmostEqual(x, 0.5, places=2)

    def test_known_value_neg_10(self):
        x = db2pow(-10)
        self.assertAlmostEqual(x, 0.1, places=2)


class Test_lin_to_db_input_datatypes(unittest.TestCase):
    def test_single(self):
        x = lin2db(1)
        self.assertEqual(x.ndim, 0)
        self.assertAlmostEqual(x, 0.0, places=6)

    def test_tuple(self):
        x = lin2db((1, 0.1))
        self.assertEqual(x.ndim, 1)
        self.assertAlmostEqual(x[0],   0.0, places=6)
        self.assertAlmostEqual(x[1], -20.0, places=6)

    def test_list(self):
        x = lin2db([10, 1])
        self.assertEqual(x.ndim, 1)
        self.assertAlmostEqual(x[0],  20.0, places=6)
        self.assertAlmostEqual(x[1],   0.0, places=6)

    def test_np_ndim_1(self):
        x = lin2db(np.ones(10))
        self.assertEqual(x.ndim, 1)
        self.assertTrue((x <  0.0001).all())                    # noqa: E222
        self.assertTrue((x > -0.0001).all())

    def test_np_ndim_2_10x4(self):
        x = lin2db(np.ones((10, 4)))
        self.assertEqual(x.ndim, 2)
        self.assertTrue((x <  0.0001).all())                    # noqa: E222
        self.assertTrue((x > -0.0001).all())

    def test_np_ndim_2_4x10(self):
        x = lin2db(np.ones((4, 10)))
        self.assertEqual(x.ndim, 2)
        self.assertTrue((x <  0.0001).all())                    # noqa: E222
        self.assertTrue((x > -0.0001).all())


class Test_db_to_lin_input_datatypes(unittest.TestCase):
    def test_single(self):
        x = db2lin(0)
        self.assertEqual(x.ndim, 0)
        self.assertAlmostEqual(x, 1.0, places=6)

    def test_tuple(self):
        x = db2lin((40, -40))
        self.assertEqual(x.ndim, 1)
        self.assertAlmostEqual(x[0],  100.0,  places=6)
        self.assertAlmostEqual(x[1],    0.01, places=6)

    def test_list(self):
        x = db2lin([-20, 60])
        self.assertEqual(x.ndim, 1)
        self.assertAlmostEqual(x[0],    0.1, places=6)
        self.assertAlmostEqual(x[1], 1000.0, places=6)

    def test_np_ndim_1(self):
        x = db2lin(np.zeros(10))
        self.assertEqual(x.ndim, 1)
        self.assertTrue((x < 1.0001).all())
        self.assertTrue((x > 0.9999).all())

    def test_np_ndim_2_10x4(self):
        x = db2lin(np.zeros((10, 4)))
        self.assertEqual(x.ndim, 2)
        self.assertTrue((x < 1.0001).all())
        self.assertTrue((x > 0.9999).all())

    def test_np_ndim_2_4x10(self):
        x = db2lin(np.zeros((4, 10)))
        self.assertEqual(x.ndim, 2)
        self.assertTrue((x < 1.0001).all())
        self.assertTrue((x > 0.9999).all())


if __name__ == "__main__":
    logging.basicConfig(
        format="%(levelname)-8s: %(module)s.%(funcName)-15s %(message)s",
        level="DEBUG",
        )
    # some libraries are noisy in DEBUG
    logging.getLogger("matplotlib").setLevel(logging.INFO)
    logging.getLogger("PIL").setLevel(logging.WARNING)

    # from command line:
    # $ python -m unittest -v zignal/tests/<filename>.py
    unittest.main(verbosity=2)



'''
Created on 8 Apr 2020

@author: Ronny Andersson (ronny@andersson.tk)
@copyright: (c) 2020 Ronny Andersson
@license: MIT
'''

# Standard library
import logging

# Third party
import matplotlib.pyplot as plt
import numpy as np

# Internal
from zignal import lin2db


def cic(N=1, D=10, f=None):
    """
    Cascaded Integrator-Comb filter, CIC filter.

    N     order
    D     decimation
    f     frequencies to evaluate

    The equivalent first order (moving average) FIR filter has D
    number of coefficients all set to 1/D

    For example, cic with N=1 D=10 would have FIR B coefficients set to

        >>> B = np.ones(10)
        >>> B = B / len(B)
        >>> B
        array([ 0.1,  0.1,  0.1,  0.1,  0.1,  0.1,  0.1,  0.1,  0.1,  0.1])

    """

    if f is None:
        f = np.linspace(10**-6, 0.5, num=50000)

    # The theoretical frequency response. Only the magnitude response
    # is calculated here.
    m = np.power(np.absolute(np.sin(np.pi*f*D)/np.sin(np.pi*f)), N)

    # normalise so that range is 0..1
    m = m/(D**N)

    # convert to decibel
    db = lin2db(m)

    return f, db


if __name__ == '__main__':
    logging.basicConfig(
        format="%(levelname)-8s: %(module)s.%(funcName)-15s %(message)s",
        level="DEBUG",
        )
    # some libraries are noisy in DEBUG
    logging.getLogger("matplotlib").setLevel(logging.INFO)
    logging.getLogger("PIL").setLevel(logging.WARNING)

    # Demo plot, 1st to 4th order cic filters with decimation set to 10
    plt.figure(1)
    plt.plot(*cic(1, 10), ls="-",  label="1st order")
    plt.plot(*cic(2, 10), ls="--", label="2nd order")
    plt.plot(*cic(3, 10), ls="-.", label="3rd order")
    plt.plot(*cic(4, 10), ls=":",  label="4th order")
    plt.ylim(-80, 3)
    plt.xlabel("Normalised frequency")
    plt.ylabel("Magnitude [dB]")
    plt.legend()
    plt.grid()
    plt.show()

    print('++ End of script ++')



'''
Created on 28 Feb 2014

@author: Ronny Andersson (ronny@andersson.tk)
@copyright: (c) 2014 Ronny Andersson
@license: MIT
'''

# Standard library
import logging
import unittest

# Third party
import numpy as np

# Internal
from zignal import Audio


class Test_EmptyConstructor(unittest.TestCase):
    def setUp(self):
        self.x = Audio()

    def test_default_constructor(self):
        self.assertAlmostEqual(self.x.fs, 96000, places=7)
        self.assertEqual(self.x.ch, 0)
        self.assertEqual(self.x.nofsamples, 0)
        self.assertEqual(self.x.duration, 0)
        self.assertIsInstance(self.x.samples, np.ndarray)

    def test_str_method(self):
        self.assertIsInstance(self.x.__str__(), str)

    def test_empty_comment(self):
        self.assertSequenceEqual(self.x.comment(), '')

    def test_add_comment(self):
        self.assertSequenceEqual(self.x.comment(), '')

        s = 'This is a comment\nwith a line break'
        self.x.comment(comment=s)
        self.assertSequenceEqual(self.x.comment(), s, msg="\n"+str(self.x))


class Test_ConstructorChannels(unittest.TestCase):
    def setUp(self):
        self.x = Audio(channels=4)

    def test_str_method(self):
        self.assertIsInstance(self.x.__str__(), str)

    def test_channels(self):
        self.assertEqual(self.x.ch, 4)
        self.assertEqual(len(self.x), 0)

    def test_RMS_is_nan(self):
        self.assertTrue(np.isnan(self.x.rms()).all(), msg=self.x.rms())

    def test_peak_is_nan(self):
        peak, idx = self.x.peak()
        self.assertTrue(np.isnan(peak).all(), msg=str(peak))
        self.assertTrue((idx == 0).all(), msg=str(idx))

    def test_crestfactor_is_nan(self):
        self.assertTrue(np.isnan(self.x.crest_factor()).all(), msg=self.x.crest_factor())


class Test_ConstructorDuration(unittest.TestCase):
    def test_set_samples(self):
        x = Audio(nofsamples=300, fs=600)
        self.assertAlmostEqual(x.duration, 0.5, places=7, msg="\n"+str(x))

    def test_set_duration(self):
        x = Audio(duration=1.5, fs=600)
        self.assertEqual(len(x), 900, msg="\n"+str(x))

    def test_set_duration_and_channels(self):
        x = Audio(duration=1.5, fs=600, channels=5)
        self.assertEqual(len(x), 900, msg="\n"+str(x))

    def test_set_duration_and_samples(self):
        self.assertRaises(AssertionError, Audio, nofsamples=10, duration=1.1)


if __name__ == "__main__":
    logging.basicConfig(
        format="%(levelname)-8s: %(module)s.%(funcName)-15s %(message)s",
        level="DEBUG",
        )
    # some libraries are noisy in DEBUG
    logging.getLogger("matplotlib").setLevel(logging.INFO)
    logging.getLogger("PIL").setLevel(logging.WARNING)

    # from command line:
    # $ python -m unittest -v zignal/tests/<filename>.py
    unittest.main(verbosity=2)



'''
Created on 21 Jun 2015

@author: Ronny Andersson (ronny@andersson.tk)
@copyright: (c) 2015 Ronny Andersson
@license: MIT
'''

# Standard library
import logging
import unittest

# Internal
from zignal.filters import linearfilter


class Test_Filter(unittest.TestCase):
    def verify_coefficients(self, f, B, A):
        for i, v in enumerate(f.get_coefficients()[0]):
            self.assertAlmostEqual(v, B[i], places=7)

        for i, v in enumerate(f.get_coefficients()[1]):
            self.assertAlmostEqual(v, A[i], places=7)

    def test_get_cefficients(self):
        B = (0.1, 0.2, 0.3)
        A = (0.4, 0.5, 0.6)
        f = linearfilter.Filter(B, A, fs=48000)
        #print(f)

        self.verify_coefficients(f, B, A)

    def test_get_feed_forward(self):
        B = (0.1, 0.2, 0.3)
        A = (0.4, 0.5, 0.6)
        f = linearfilter.Filter(B, A, fs=48000)
        #print(f)

        BB = f.get_feed_forward()

        self.verify_coefficients(f, BB, A)

    def test_get_feed_back(self):
        B = (0.1, 0.2, 0.3)
        A = (0.4, 0.5, 0.6)
        f = linearfilter.Filter(B, A, fs=48000)
        #print(f)

        AA = f.get_feed_back()

        self.verify_coefficients(f, B, AA)

    def test_normalise(self):
        B = (0.1, 0.2, 0.3)
        A = (0.4, 0.5, 0.6)
        f = linearfilter.Filter(B, A, fs=48000)
        #print(f)

        f.normalise()
        unused_BB, AA = f.get_coefficients()
        self.assertAlmostEqual(AA[0], 1.0, places=7)

    def test_set_cefficients(self):
        B = (0.7, 0.8, 0.9)
        A = (1.1, 1.2, 1.3)
        f = linearfilter.Filter(fs=48000)
        f.set_coefficients(B, A)
        #print(f)

        self.verify_coefficients(f, B, A)

    def test_str_method(self):
        f = linearfilter.Filter()
        self.assertIsInstance(f.__str__(), str)


class Test_normalised_frequency(unittest.TestCase):
    def test_full_samplerate(self):
        self.assertAlmostEqual(linearfilter.normalised_frequency(f0=44100, fs=44100), 2.0, places=7)

    def test_half_samplerate(self):
        self.assertAlmostEqual(linearfilter.normalised_frequency(f0=1000,  fs=2000),  1.0, places=7)

    def test_quarter_samplerate(self):
        self.assertAlmostEqual(linearfilter.normalised_frequency(f0=24000, fs=96000), 0.5, places=7)


if __name__ == "__main__":
    logging.basicConfig(
        format="%(levelname)-8s: %(module)s.%(funcName)-15s %(message)s",
        level="DEBUG",
        )
    # some libraries are noisy in DEBUG
    logging.getLogger("matplotlib").setLevel(logging.INFO)
    logging.getLogger("PIL").setLevel(logging.WARNING)

    # from command line:
    # $ python -m unittest -v zignal/tests/<filename>.py
    unittest.main(verbosity=2)



'''
Created on 9 Apr 2020

@author: Ronny Andersson (ronny@andersson.tk)
@copyright: (c) 2020 Ronny Andersson
@license: MIT
'''

# Standard library
import logging

# Third party
import matplotlib.pyplot as plt
import numpy as np

# Internal
import zignal

if __name__ == '__main__':
    logging.basicConfig(
        format="%(levelname)-8s: %(module)s.%(funcName)-15s %(message)s",
        level="DEBUG",
        )
    # some libraries are noisy in DEBUG
    logging.getLogger("matplotlib").setLevel(logging.INFO)
    logging.getLogger("PIL").setLevel(logging.WARNING)

    # Create an FIR filter with n coefficients set to 1/n
    # Plot this and compare it with a cic filter with D=n

    n = 10

    B = np.ones(n)
    B = B / len(B)

    f = zignal.filters.linearfilter.FIR(B=B, fs=1)
    print(f)

    #f.plot_impulse_resp()
    f.plot_pole_zero()
    #f.plot_mag_phase(unwrap=True)

    plt.figure(1)

    plt.plot(*zignal.filters.cic.cic(1, n),
             ls="-",  lw=2.0, c="b", label="CIC 1st order")

    plt.plot(*f.magnitude_resp(frequencies=50000),
             ls="--", lw=2.0, c="r", label="FIR comb")

    plt.ylim(-80, 3)
    plt.xlabel("Normalised frequency")
    plt.ylabel("Magnitude [dB]")
    plt.legend()
    plt.grid()
    plt.show()

    print('++ End of script ++')



'''
Created on 21 Feb 2015

@author: Ronny Andersson (ronny@andersson.tk)
@copyright: (c) 2015 Ronny Andersson
@license: MIT
'''

# Standard library
import logging
import unittest

# Internal
from zignal.music import spn


class Test_key2index(unittest.TestCase):
    def test_increasing(self):
        self.assertEqual(spn.key2index('A0'),   1)
        self.assertEqual(spn.key2index('A#0'),  2)
        self.assertEqual(spn.key2index('B0'),   3)
        self.assertEqual(spn.key2index('C1'),   4)
        self.assertEqual(spn.key2index('C#1'),  5)
        self.assertEqual(spn.key2index('D1'),   6)
        self.assertEqual(spn.key2index('D#1'),  7)
        self.assertEqual(spn.key2index('E1'),   8)
        self.assertEqual(spn.key2index('F1'),   9)
        self.assertEqual(spn.key2index('F#1'),  10)
        self.assertEqual(spn.key2index('G1'),   11)
        self.assertEqual(spn.key2index('G#1'),  12)
        self.assertEqual(spn.key2index('A1'),   13)
        self.assertEqual(spn.key2index('A#1'),  14)
        self.assertEqual(spn.key2index('B1'),   15)
        self.assertEqual(spn.key2index('C2'),   16)

    def test_flat(self):
        self.assertEqual(spn.key2index('Abbb4'), 46)
        self.assertEqual(spn.key2index('Abb4'),  47)
        self.assertEqual(spn.key2index('Ab4'),   48)

    def test_standard(self):
        self.assertEqual(spn.key2index('A4'),   49)

    def test_sharp(self):
        self.assertEqual(spn.key2index('A#4'),  50)
        self.assertEqual(spn.key2index('Ax4'),  51)
        self.assertEqual(spn.key2index('A#x4'), 52)

    def test_lowest_piano_key(self):
        self.assertEqual(spn.key2index('A0'),   1)

    def test_beyond_lowest_piano_key(self):
        self.assertEqual(spn.key2index('Ab0'),       0)
        self.assertEqual(spn.key2index('Abb0'),     -1)
        self.assertEqual(spn.key2index('C0'),       -8)
        self.assertEqual(spn.key2index('A-1'),      -11)
        self.assertEqual(spn.key2index('Ab-10'),    -10*12)

    def test_regexp_H(self):
        # H is sometimes used instead of B
        self.assertRaises(AssertionError,   spn.key2index, 'H4')

    def test_C_flat(self):
        # C-flat and B-sharp issues
        self.assertEqual(spn.key2index('Cb4'), spn.key2index('B3'))
        self.assertEqual(spn.key2index('B#4'), spn.key2index('C5'))

    def test_regexp_incomplete(self):
        self.assertRaises(ValueError,       spn.key2index, 'B')
        self.assertRaises(ValueError,       spn.key2index, '7')
        self.assertRaises(ValueError,       spn.key2index, '#')
        self.assertRaises(ValueError,       spn.key2index, 'x')
        self.assertRaises(ValueError,       spn.key2index, 'bb')

    def test_regexp_invalid(self):
        self.assertRaises(AssertionError,   spn.key2index, 'asdfghj')
        self.assertRaises(AssertionError,   spn.key2index, '213ewq')
        self.assertRaises(AssertionError,   spn.key2index, 'c4')
        self.assertRaises(ValueError,       spn.key2index, 'Cxx4')
        self.assertRaises(ValueError,       spn.key2index, 'C##4')
        self.assertRaises(ValueError,       spn.key2index, 'C###4')
        self.assertRaises(ValueError,       spn.key2index, 'Cbbbb4')

    def test_regexp_empty(self):
        self.assertRaises(ValueError,       spn.key2index, '')
        self.assertRaises(AssertionError,   spn.key2index, ' ')


class Test_index2key(unittest.TestCase):
    def test_known_values(self):
        self.assertEqual(spn.index2key(49), 'A4')
        self.assertEqual(spn.index2key(1),  'A0')
        self.assertEqual(spn.index2key(-8), 'C0')
        self.assertEqual(spn.index2key(51), 'B4')
        self.assertEqual(spn.index2key(11), 'G1')

    def test_increasing(self):
        self.assertEqual(spn.index2key(1),  'A0')
        self.assertEqual(spn.index2key(2),  'A#0')
        self.assertEqual(spn.index2key(3),  'B0')
        self.assertEqual(spn.index2key(4),  'C1')
        self.assertEqual(spn.index2key(5),  'C#1')
        self.assertEqual(spn.index2key(6),  'D1')
        self.assertEqual(spn.index2key(7),  'D#1')
        self.assertEqual(spn.index2key(8),  'E1')
        self.assertEqual(spn.index2key(9),  'F1')
        self.assertEqual(spn.index2key(10), 'F#1')
        self.assertEqual(spn.index2key(11), 'G1')
        self.assertEqual(spn.index2key(12), 'G#1')
        self.assertEqual(spn.index2key(13), 'A1')
        self.assertEqual(spn.index2key(14), 'A#1')
        self.assertEqual(spn.index2key(15), 'B1')
        self.assertEqual(spn.index2key(16), 'C2')

    def test_argument_float(self):
        self.assertRaises(AssertionError,   spn.index2key,   1.0)
        self.assertRaises(AssertionError,   spn.index2key,  -1.0)
        self.assertRaises(AssertionError,   spn.index2key,   2.3)
        self.assertRaises(AssertionError,   spn.index2key,  -0.01)

    def test_argument_string(self):
        self.assertRaises(AssertionError,   spn.index2key,  'A4')
        self.assertRaises(AssertionError,   spn.index2key,  '4')


if __name__ == "__main__":
    logging.basicConfig(
        format="%(levelname)-8s: %(module)s.%(funcName)-15s %(message)s",
        level="DEBUG",
        )
    # some libraries are noisy in DEBUG
    logging.getLogger("matplotlib").setLevel(logging.INFO)
    logging.getLogger("PIL").setLevel(logging.WARNING)

    # from command line:
    # $ python -m unittest -v zignal/tests/<filename>.py
    unittest.main(verbosity=2)



'''
Created on 24 Feb 2015

@author: Ronny Andersson (ronny@andersson.tk)
@copyright: (c) 2015 Ronny Andersson
@license: MIT
'''

# Standard library
import logging
import unittest

# Internal
from zignal.music import scales


class Test_midi_scales(unittest.TestCase):
    # Benson, DJ. (2006). Music: A Mathematical Offering. Cambridge University Press.
    # http://homepages.abdn.ac.uk/mth192/pages/html/maths-music.html
    def test_freq2key_quantise(self):
        # 70 466.164
        # 69 440.00
        # 68 415.305
        self.assertAlmostEqual(scales.midi_freq2key(416.4, quantise=True), 68, places=7)
        self.assertAlmostEqual(scales.midi_freq2key(438.0, quantise=True), 69, places=7)
        self.assertAlmostEqual(scales.midi_freq2key(441.0, quantise=True), 69, places=7)
        self.assertAlmostEqual(scales.midi_freq2key(442.0, quantise=True), 69, places=7)
        self.assertAlmostEqual(scales.midi_freq2key(452.1, quantise=True), 69, places=7)
        self.assertAlmostEqual(scales.midi_freq2key(453.1, quantise=True), 70, places=7)
        self.assertAlmostEqual(scales.midi_freq2key(460.0, quantise=True), 70, places=7)
        self.assertAlmostEqual(scales.midi_freq2key(470.0, quantise=True), 70, places=7)

    def test_key2freq(self):
        self.assertAlmostEqual(scales.midi_key2freq(69), 440.0,     places=7)
        self.assertAlmostEqual(scales.midi_key2freq(81), 880.0,     places=7)
        self.assertAlmostEqual(scales.midi_key2freq(21), 27.5,      places=7)
        self.assertAlmostEqual(scales.midi_key2freq(43), 97.9989,   places=4)

    def test_freq2key(self):
        self.assertAlmostEqual(scales.midi_freq2key(440), 69.0,     places=7)
        self.assertAlmostEqual(scales.midi_freq2key(880), 81.0,     places=7)

    def test_key2freq_tuning(self):
        self.assertAlmostEqual(scales.midi_key2freq(69, tuning=450), 450.0, places=7)
        self.assertAlmostEqual(scales.midi_key2freq(81, tuning=450), 900.0, places=7)
        self.assertAlmostEqual(scales.midi_key2freq(21, tuning=400),  25.0, places=7)

    def test_freq2key_tuning(self):
        self.assertAlmostEqual(scales.midi_freq2key(450, tuning=450), 69.0, places=7)
        self.assertAlmostEqual(scales.midi_freq2key(900, tuning=450), 81.0, places=7)

    def test_back2back_key(self):
        self.assertAlmostEqual(
            scales.midi_key2freq(scales.midi_freq2key(1234)),    1234,   places=7)
        self.assertAlmostEqual(
            scales.midi_key2freq(scales.midi_freq2key(45.67)),   45.67,  places=7)

    def test_back2back_freq(self):
        self.assertAlmostEqual(
            scales.midi_freq2key(scales.midi_key2freq(76.543)),  76.543, places=7)
        self.assertAlmostEqual(
            scales.midi_freq2key(scales.midi_key2freq(124)),     124,    places=7)


class Test_piano_note_to_freq(unittest.TestCase):
    def test_octaves(self):
        self.assertAlmostEqual(scales.piano_note2freq('A2'), 110.0,     places=7)
        self.assertAlmostEqual(scales.piano_note2freq('A3'), 220.0,     places=7)
        self.assertAlmostEqual(scales.piano_note2freq('A4'), 440.0,     places=7)
        self.assertAlmostEqual(scales.piano_note2freq('A5'), 880.0,     places=7)
        self.assertAlmostEqual(scales.piano_note2freq('A6'), 1760.0,    places=7)

    def test_values(self):
        self.assertAlmostEqual(scales.piano_note2freq('C6'), 1046.50,   places=2)
        self.assertAlmostEqual(scales.piano_note2freq('D1'), 36.7081,   places=4)


class Test_piano_freq_to_note(unittest.TestCase):
    def test_values(self):
        self.assertEqual(scales.piano_freq2note(1046.50),   'C6')
        self.assertEqual(scales.piano_freq2note(36.7051),   'D1')
        self.assertEqual(scales.piano_freq2note(440),       'A4')

    def test_quantise(self):
        self.assertEqual(scales.piano_freq2note(435.00), 'A4')
        self.assertEqual(scales.piano_freq2note(439.00), 'A4')
        self.assertEqual(scales.piano_freq2note(440.00), 'A4')
        self.assertEqual(scales.piano_freq2note(441.00), 'A4')
        self.assertEqual(scales.piano_freq2note(447.00), 'A4')


class Test_piano(unittest.TestCase):
    def test_back2back_key(self):
        self.assertAlmostEqual(
            scales.piano_key2freq(scales.piano_freq2key(100)),   100,   places=7)
        self.assertAlmostEqual(
            scales.piano_key2freq(scales.piano_freq2key(32)),    32,    places=7)
        self.assertAlmostEqual(
            scales.piano_key2freq(scales.piano_freq2key(997)),   997,   places=7)
        self.assertAlmostEqual(
            scales.piano_key2freq(scales.piano_freq2key(12345)), 12345, places=7)
        self.assertAlmostEqual(
            scales.piano_key2freq(scales.piano_freq2key(4.563)), 4.563, places=7)

    def test_back2back_freq(self):
        self.assertAlmostEqual(
            scales.piano_freq2key(scales.piano_key2freq(10)), 10,       places=7)
        self.assertAlmostEqual(
            scales.piano_freq2key(scales.piano_key2freq(49)), 49,       places=7)
        self.assertAlmostEqual(
            scales.piano_freq2key(scales.piano_key2freq(30.3)), 30.3,   places=7)

    def test_back2back_freq_quantised(self):
        self.assertAlmostEqual(scales.piano_freq2key(scales.piano_key2freq(10.2),
                                                     quantise=True), 10, places=7)
        self.assertAlmostEqual(scales.piano_freq2key(scales.piano_key2freq(34.678),
                                                     quantise=True), 35, places=7)


if __name__ == "__main__":
    logging.basicConfig(
        format="%(levelname)-8s: %(module)s.%(funcName)-15s %(message)s",
        level="DEBUG",
        )
    # some libraries are noisy in DEBUG
    logging.getLogger("matplotlib").setLevel(logging.INFO)
    logging.getLogger("PIL").setLevel(logging.WARNING)

    # from command line:
    # $ python -m unittest -v zignal/tests/<filename>.py
    unittest.main(verbosity=2)



'''
Created on 12 Apr 2020

@author: Ronny Andersson (ronny@andersson.tk)
@copyright: (c) 2020 Ronny Andersson
@license: MIT

Demo of how to iterate over an instance of the Audio class, for chunk-based
processing. Typically the chunks have a size that is a power of two, for
example 256, 1024 or 4096. In this example the chunk size is set to 1000
for simplicity in the plots. The sample rate in this example is also set to
a value that enhances the effect of the example, since hera a chunk equals
to one second of data.
'''

# Standard library
import logging

# Third party
import matplotlib.pyplot as plt
import numpy as np

# Internal
import zignal

if __name__ == '__main__':
    logging.basicConfig(
        format="%(levelname)-8s: %(module)s.%(funcName)-15s %(message)s",
        level="DEBUG",
        )
    # some libraries are noisy in DEBUG
    logging.getLogger("matplotlib").setLevel(logging.INFO)
    logging.getLogger("PIL").setLevel(logging.WARNING)

    fs = 1000

    # Create various ramp signals, to visualise the chunks better. Not real
    # audio, but shows in a plot what the chunks look like
    a1 = zignal.Audio(fs=fs, initialdata=np.linspace(0, 1,  num=(1000//2)))
    a2 = zignal.Audio(fs=fs, initialdata=np.linspace(0, -1, num=(1000*1)+500))
    a3 = zignal.Audio(fs=fs, initialdata=np.linspace(0, 1,  num=(1000*2)+200))

    a = zignal.Audio(fs=fs)
    a.append(a1, a2, a3)
    print(a)

    # We now have 2.2 seconds of audio in three channels. This does not add up
    # to even chunk sizes, so padding will have to be done in order to iterate.
    #
    # Three (3) chunks are expected.
    for val in a.iter_chunks(chunksize=1000):
        print("------------------------------------------------")
        print("shape of data in chunk: %s" % str(val.shape))
        print(val)

        plt.figure(1)
        plt.plot(val[:, 0], ls="-",  label="a1")
        plt.plot(val[:, 1], ls="--", label="a2")
        plt.plot(val[:, 2], ls="-.", label="a3")
        plt.grid()
        plt.ylim(-1.1, 1.1)
        plt.xlabel("samples in chunk")
        plt.ylabel("magnitude [lin]")
        plt.legend(loc="upper right")
        plt.show()

    # We can pad beforehand if we know how many samples are missing, then no
    # padding will occur inside the iterator
    b = a.copy()

    b.gain(-20)   # just to get a debug logging entry
    b.pad(nofsamples=800)
    print(b)
    for val in b.iter_chunks(chunksize=1000):
        print("------------------------------------------------")
        print("shape of data in chunk: %s" % str(val.shape))
        print(val)

    print('-- Done --')



'''
Created on 16 Feb 2015

@author: Ronny Andersson (ronny@andersson.tk)
@copyright: (c) 2015 Ronny Andersson
@license: MIT
'''

# Standard library
import logging

# Internal
from zignal.audio import Audio, Noise, Sinetone, SquareWave

if __name__ == '__main__':
    logging.basicConfig(
        format="%(levelname)-8s: %(module)s.%(funcName)-15s %(message)s",
        level="DEBUG",
        )
    # some libraries are noisy in DEBUG
    logging.getLogger("matplotlib").setLevel(logging.INFO)
    logging.getLogger("PIL").setLevel(logging.WARNING)

    fs  = 48000
    dur = 1.5

    x1 = Sinetone(f0=997, fs=fs, duration=dur, gaindb=0)
    x2 = Noise(fs=fs, duration=dur, gaindb=-6)
    x3 = SquareWave(f0=3000, fs=fs, duration=dur, gaindb=-20)
    x4 = Audio(fs=fs)
    x4.append(x1, x2, x3)

    print(x1)
    print(x1.peak())
    print(x1.rms())
    print(x1.crest_factor())

    print(x2)
    print(x2.peak())
    print(x2.rms())
    print(x2.crest_factor())

    print(x3)
    print(x3.peak())
    print(x3.rms())
    print(x3.crest_factor())

    print(x4)
    print(x4.peak())
    print(x4.rms())
    print(x4.crest_factor())



"""
Music package

@author: Ronny Andersson (ronny@andersson.tk)
@copyright: (c) 2016 Ronny Andersson
@license: MIT
"""

# Local folders
from . import scales, spn

__all__ = [
    'scales',
    'spn',
    ]



'''
Created on 23 Aug 2015

@author: Ronny Andersson (ronny@andersson.tk)
@copyright: (c) 2015 Ronny Andersson
@license: MIT
'''

# Standard library
import logging

# Internal
from zignal.filters.biquads import RBJ, Zolzer

if __name__ == '__main__':
    logging.basicConfig(
        format="%(levelname)-8s: %(module)s.%(funcName)-15s %(message)s",
        level="DEBUG",
        )
    # some libraries are noisy in DEBUG
    logging.getLogger("matplotlib").setLevel(logging.INFO)
    logging.getLogger("PIL").setLevel(logging.WARNING)

    gaindb  = -6
    f0      = 997
    Q       = 0.707
    fs      = 48000

    # Create a Robert Bristow-Johnson type biquad filter
    f1 = RBJ(filtertype=RBJ.Types.peak, gaindb=gaindb, f0=f0, Q=Q, fs=fs)

    f1.plot_mag_phase()
    f1.plot_pole_zero()

    # Create an Udo Zolzer type biquad filter
    f2 = Zolzer(filtertype=Zolzer.Types.peak, gaindb=gaindb, f0=f0, Q=Q, fs=fs)

    f2.plot_mag_phase()
    f2.plot_pole_zero()



'''
Created on 13 Apr 2020

@author: Ronny Andersson (ronny@andersson.tk)
@copyright: (c) 2020 Ronny Andersson
@license: MIT
'''

# Standard library
import logging

# Internal
import zignal

if __name__ == '__main__':
    logging.basicConfig(
        format="%(levelname)-8s: %(module)s.%(funcName)-15s %(message)s",
        level="DEBUG",
        )
    # some libraries are noisy in DEBUG
    logging.getLogger("matplotlib").setLevel(logging.INFO)
    logging.getLogger("PIL").setLevel(logging.WARNING)

    fs = 200
    a = zignal.Audio(fs=fs)
    a1 = zignal.Sinetone(f0=1, fs=fs, duration=1, gaindb=-6)
    a2 = zignal.Sinetone(f0=2, fs=fs, duration=1, gaindb=-6)
    a.append(a1, a2)
    print(a)

    a.plot(ch="all", marker="X")
    a.decimate(10)
    print(a)
    a.plot(ch="all", marker="X")

    print('-- Done --')



'''
Created on 16 Feb 2015

@author: Ronny Andersson (ronny@andersson.tk)
@copyright: (c) 2015 Ronny Andersson
@license: MIT
'''

# Standard library
import logging

# Internal
from zignal.audio import Audio, Noise

if __name__ == '__main__':
    logging.basicConfig(
        format="%(levelname)-8s: %(module)s.%(funcName)-15s %(message)s",
        level="DEBUG",
        )
    # some libraries are noisy in DEBUG
    logging.getLogger("matplotlib").setLevel(logging.INFO)
    logging.getLogger("PIL").setLevel(logging.WARNING)

    fs  = 96000
    dur = 1.0

    nw  = Noise(fs=fs, duration=dur, gaindb=-6, colour='white')
    np  = Noise(fs=fs, duration=dur, gaindb=-6, colour='pink')
    x   = Audio(fs=fs)
    x.append(nw, np)

    print(nw)
    print(np)
    print(x)
    x.plot_fft()



'''
Created on 15 Feb 2015

@author: Ronny Andersson (ronny@andersson.tk)
@copyright: (c) 2015 Ronny Andersson
@license: MIT
'''

# Standard library
import logging

# Internal
from zignal.audio import Sinetone

if __name__ == '__main__':
    logging.basicConfig(
        format="%(levelname)-8s: %(module)s.%(funcName)-15s %(message)s",
        level="DEBUG",
        )
    # some libraries are noisy in DEBUG
    logging.getLogger("matplotlib").setLevel(logging.INFO)
    logging.getLogger("PIL").setLevel(logging.WARNING)

    fs  = 1000
    f0  = 10
    dur = 0.1

    x = Sinetone(f0=f0, fs=fs, duration=dur, gaindb=0)
    print(x)
    x.plot(linestyle='--', marker='x', color='r', label='sine at %i Hz' % f0)

    x.set_sample_rate(500)
    print(x)
    x.plot(linestyle='-.', color='k', label='sine at %i Hz' % f0)



'''
Created on 31 Oct 2014

@author: Ronny Andersson (ronny@andersson.tk)
@copyright: (c) 2014 Ronny Andersson
@license: MIT
'''

# Standard library
import logging
import unittest

# Third party
import numpy as np

# Internal
from zignal import Audio


class Test_single_channel(unittest.TestCase):
    def setUp(self):
        self.y = np.zeros(4)

    def check_values(self, values, expected, position):
        x = Audio(fs=10, initialdata=values)

        peak, idx = x.peak()
        self.assertTrue(len(peak) == 1)
        self.assertTrue(len(idx) == 1)

        #print("index: %3i  peak: %f" % (idx, peak))
        #print(x)

        self.assertAlmostEqual(peak, expected, places=3)
        self.assertEqual(idx, position)

    def test_positive(self):
        self.y[1] =  2.0                            # noqa: E222
        self.y[2] =  2.2    # <-- peak              # noqa: E222
        self.y[3] = -1.2
        #print("init data: %s" % self.y)
        self.check_values(self.y, 2.2, 2)

    def test_negative(self):
        self.y[1] =  2.0                            # noqa: E222
        self.y[2] =  3.19                           # noqa: E222
        self.y[3] = -3.2    # <-- peak
        #print("init data: %s" % self.y)
        self.check_values(self.y, -3.2, 3)


class Test_multi_channel(unittest.TestCase):
    def setUp(self):
        self.y = np.zeros((4, 2))

    def check_values(self, values, expected, position):
        x = Audio(fs=10, initialdata=values)

        peak, idx = x.peak()
        self.assertTrue(len(peak) == 2)
        self.assertTrue(len(idx) == 2)

        #print("index: %s  peak: %s" % (idx, peak))
        #print(x)

        self.assertAlmostEqual(peak[0], expected[0], places=3)
        self.assertAlmostEqual(peak[1], expected[1], places=3)

        self.assertEqual(idx[0], position[0])
        self.assertEqual(idx[1], position[1])

    def test_positive(self):
        self.y[1][0] =  1.0                         # noqa: E222
        self.y[2][0] =  2.3     # <-- peak          # noqa: E222

        self.y[1][1] = -4.1     # <-- peak
        self.y[2][1] =  3.0                         # noqa: E222
        #print(self.y)
        self.check_values(self.y, [2.3, -4.1], [2, 1])

    def test_negative(self):
        self.y[1][0] =  1.0                         # noqa: E222
        self.y[2][0] =  2.0     # <-- peak          # noqa: E222

        self.y[0][1] = -4.0     # <-- peak
        self.y[1][1] =  3.0                         # noqa: E222
        #print(self.y)
        self.check_values(self.y, [2, -4], [2, 0])


if __name__ == "__main__":
    logging.basicConfig(
        format="%(levelname)-8s: %(module)s.%(funcName)-15s %(message)s",
        level="DEBUG",
        )
    # some libraries are noisy in DEBUG
    logging.getLogger("matplotlib").setLevel(logging.INFO)
    logging.getLogger("PIL").setLevel(logging.WARNING)

    # from command line:
    # $ python -m unittest -v zignal/tests/<filename>.py
    unittest.main(verbosity=2)



'''
Created on 15 Feb 2015

@author: Ronny Andersson (ronny@andersson.tk)
@copyright: (c) 2015 Ronny Andersson
@license: MIT
'''

# Standard library
import logging

# Internal
from zignal.audio import Audio, Noise, Sinetone

if __name__ == '__main__':
    logging.basicConfig(
        format="%(levelname)-8s: %(module)s.%(funcName)-15s %(message)s",
        level="DEBUG",
        )
    # some libraries are noisy in DEBUG
    logging.getLogger("matplotlib").setLevel(logging.INFO)
    logging.getLogger("PIL").setLevel(logging.WARNING)

    fs  = 10000
    f0  = 10
    dur = 0.1

    x1  = Sinetone(f0=f0, fs=fs, duration=dur+0.2, gaindb=0)
    xn  = Noise(fs=fs, duration=dur, gaindb=-20)
    x   = Audio(fs=fs)

    print(x)

    x.append(xn, x1)
    del x1, xn

    print(x)

    x.plot(ch='all')



'''
Created on 16 Feb 2015

This example will play some audio on the system standard sound card.

@author: Ronny Andersson (ronny@andersson.tk)
@copyright: (c) 2015 Ronny Andersson
@license: MIT
'''

# Standard library
import logging

# Internal
import zignal.sndcard


def ex_1_play():
    # The recommended way of creating and using a sndcard instance is by using the
    # "with" statement. This will make sure that the instance is closed correctly
    # after usage. See http://effbot.org/zone/python-with-statement.htm
    #
    # This example plays the audio on the default device

    fs = 44100
    x  = zignal.Sinetone(f0=400, fs=fs, duration=1.5, gaindb=-12)
    x2 = zignal.Sinetone(f0=900, fs=fs, duration=1.5, gaindb=-18)

    x.append(x2)
    x.convert_to_float(targetbits=32)

    with zignal.sndcard.PA(device_in='default', device_out='default') as snd:
        # using an assert here helps PyDev in eclipse when pressing ctrl+space for autocomplete
        assert isinstance(snd, zignal.sndcard._Device)
        snd.play(x)


def ex_2_play():
    # Another way of using a sndcard is by first creating an instance and
    # manually calling the open() function. The close() function *must* be
    # called in a controlled fashion. This usually means that the usage is
    # wrapped in a try-except-finally clause.

    fs = 44100
    x  = zignal.Sinetone(f0=700, fs=fs, duration=1.0, gaindb=-24)
    xn = zignal.Noise(channels=1, fs=fs, duration=1.0, gaindb=-12, colour='pink')
    x.append(xn)
    x.convert_to_integer(targetbits=16)

    snd = zignal.sndcard.PA()
    print(snd)

    snd.open()
    try:
        snd.play(x)
    finally:
        snd.close()


def ex_3_play_rec():
    # Play and record at the same time

    fs = 44100
    x  = zignal.Sinetone(f0=500, fs=fs, duration=1.5, gaindb=-12)
    x.convert_to_float(targetbits=32)

    with zignal.sndcard.PA(device_in='default', device_out='default') as snd:
        # using an assert here helps PyDev in eclipse when pressing ctrl+space for autocomplete
        assert isinstance(snd, (zignal.sndcard.PA))
        y = snd.play_rec(x, frames_per_buffer=32)

    print(y)
    y.plot()


def ex_4_rec():
    # Record

    fs = 44100
    with zignal.sndcard.PA(device_in='default') as snd:
        # using an assert here helps PyDev in eclipse when pressing ctrl+space for autocomplete
        assert isinstance(snd, (zignal.sndcard.PA))
        print("recording...")
        y = snd.rec(duration=3.5, channels=1, fs=fs)

    print(y)
    y.plot()


if __name__ == '__main__':
    logging.basicConfig(
        format="%(levelname)-8s: %(module)s.%(funcName)-15s %(message)s",
        level="DEBUG",
        )
    # some libraries are noisy in DEBUG
    logging.getLogger("matplotlib").setLevel(logging.INFO)
    logging.getLogger("PIL").setLevel(logging.WARNING)

    ex_1_play()
    ex_2_play()
    ex_3_play_rec()
    ex_4_rec()

    print('++ End of script ++')



"""
Filters package

@author: Ronny Andersson (ronny@andersson.tk)
@copyright: (c) 2014 Ronny Andersson
@license: MIT
"""

# Local folders
from . import biquads, cic, linearfilter

__all__ = [
    'biquads',
    'cic',
    'linearfilter',
    ]



'''
Created on 16 Feb 2015

@author: Ronny Andersson (ronny@andersson.tk)
@copyright: (c) 2015 Ronny Andersson
@license: MIT
'''

# Standard library
import logging

# Internal
from zignal.audio import Audio, FourierSeries

if __name__ == '__main__':
    logging.basicConfig(
        format="%(levelname)-8s: %(module)s.%(funcName)-15s %(message)s",
        level="DEBUG",
        )
    # some libraries are noisy in DEBUG
    logging.getLogger("matplotlib").setLevel(logging.INFO)
    logging.getLogger("PIL").setLevel(logging.WARNING)

    dur = 0.1
    f0  = 20

    k   = Audio()
    x0  = FourierSeries(f0=f0, duration=dur, harmonics=0,  gaindb=-15)  # fundamental + 0 harmonics
    x1  = FourierSeries(f0=f0, duration=dur, harmonics=1,  gaindb=-15)  # fundamental + 1 harmonics
    x2  = FourierSeries(f0=f0, duration=dur, harmonics=2,  gaindb=-9)   # fundamental + 2 harmonics
    x3  = FourierSeries(f0=f0, duration=dur, harmonics=3,  gaindb=-9)   # ...
    x4  = FourierSeries(f0=f0, duration=dur, harmonics=4,  gaindb=-3)
    x5  = FourierSeries(f0=f0, duration=dur, harmonics=5,  gaindb=-3)
    x60 = FourierSeries(f0=f0, duration=dur, harmonics=60, gaindb=0)    # fundamental + 60 harmonics

    k.append(x0, x1, x2, x3, x4, x5, x60)
    print(k)
    k.plot(ch='all')



'''
Created on 26 Oct 2014

@author: Ronny Andersson (ronny@andersson.tk)
@copyright: (c) 2014 Ronny Andersson
@license: MIT
'''

# Standard library
import logging
import unittest

# Internal
from zignal import Sinetone, SquareWave


class Test_Sinetone(unittest.TestCase):
    def test_endpoint(self):
        fs = 100
        f0 = 1
        x = Sinetone(f0=f0, fs=fs, duration=1, gaindb=20)
        #print(x)
        #print(x.samples[-5:, :])
        # We've created a one period sine. The last sample should not be
        # very close to zero. If it were, then a concatenation of two
        # sines would mean that we have one zero value too many at the
        # concatenation point --> discontinuity
        self.assertNotAlmostEqual(float(x.samples[-1, 0]), 0.0, places=5)

    def test_center_frequency(self):
        fs = 48000
        f0 = 997
        x = Sinetone(f0=f0, fs=fs, duration=2, gaindb=20)
        #print(x)
        freq, mag = x.fft(window="rectangular")

        self.assertAlmostEqual(freq[mag.argmax()], f0, places=7)


class Test_SetSampleRate(unittest.TestCase):
    def setUp(self):
        self.fs     = 1000
        self.dur    = 2.0
        self.f0     = 100
        self.x      = Sinetone(f0=self.f0, fs=self.fs, duration=self.dur, gaindb=-10)

        #print(self.id())
        #print('Before:\n%s' % self.x)

    def test_duration_fs_up_2_5(self):
        self.assertAlmostEqual(self.x.duration, self.dur, places=5)

        self.x.set_sample_rate(self.fs*2.5)
        #print('After:\n%s' % self.x)

        self.assertAlmostEqual(self.x.duration, self.dur/2.5, places=5)

    def test_duration_fs_down_2_5(self):
        self.assertAlmostEqual(self.x.duration, self.dur, places=5)

        self.x.set_sample_rate(self.fs/2.5)
        #print('After:\n%s' % self.x)

        self.assertAlmostEqual(self.x.duration, self.dur*2.5, places=5)

    def test_frequency_fs_down_3(self):
        self.assertAlmostEqual(self.x.f0, self.f0, places=5)

        self.x.set_sample_rate(self.fs/3)
        #print('After:\n%s' % self.x)

        self.assertAlmostEqual(self.x.f0, self.f0/3, places=5)

    def test_frequency_fs_up_3(self):
        self.assertAlmostEqual(self.x.f0, self.f0, places=5)

        self.x.set_sample_rate(self.fs*3)
        #print('After:\n%s' % self.x)

        self.assertAlmostEqual(self.x.f0, self.f0*3, places=5)


class Test_SetSampleRate_Square(Test_SetSampleRate):
    def setUp(self):
        self.fs     = 10000
        self.dur    = 3.0
        self.f0     = 500
        self.x      = SquareWave(f0=self.f0, fs=self.fs, duration=self.dur, gaindb=-10)


if __name__ == "__main__":
    logging.basicConfig(
        format="%(levelname)-8s: %(module)s.%(funcName)-15s %(message)s",
        level="DEBUG",
        )
    # some libraries are noisy in DEBUG
    logging.getLogger("matplotlib").setLevel(logging.INFO)
    logging.getLogger("PIL").setLevel(logging.WARNING)

    # from command line:
    # $ python -m unittest -v zignal/tests/<filename>.py
    unittest.main(verbosity=2)



'''
Created on 15 Feb 2015

@author: Ronny Andersson (ronny@andersson.tk)
@copyright: (c) 2015 Ronny Andersson
@license: MIT
'''

# Standard library
import logging

# Internal
from zignal.audio import Noise, Sinetone

if __name__ == '__main__':
    logging.basicConfig(
        format="%(levelname)-8s: %(module)s.%(funcName)-15s %(message)s",
        level="DEBUG",
        )
    # some libraries are noisy in DEBUG
    logging.getLogger("matplotlib").setLevel(logging.INFO)
    logging.getLogger("PIL").setLevel(logging.WARNING)

    fs  = 96000
    f0  = 997
    dur = 2.5

    x   = Sinetone(f0=f0, fs=fs, duration=dur, gaindb=0)
    n   = Noise(channels=1, fs=fs, duration=dur, gaindb=-50)
    print(x)
    print(n)

    x.samples = x.samples+n.samples

    x.plot_fft(window='hamming')



'''
Created on 15 Feb 2015

@author: Ronny Andersson (ronny@andersson.tk)
@copyright: (c) 2015 Ronny Andersson
@license: MIT
'''

# Standard library
import logging

# Internal
from zignal.audio import Audio

if __name__ == '__main__':
    logging.basicConfig(
        format="%(levelname)-8s: %(module)s.%(funcName)-15s %(message)s",
        level="DEBUG",
        )
    # some libraries are noisy in DEBUG
    logging.getLogger("matplotlib").setLevel(logging.INFO)
    logging.getLogger("PIL").setLevel(logging.WARNING)

    x = Audio()
    print(x)

    x.comment(comment="We can add comments\nthat spans multiple lines.")

    print(x.comment())
    print(x)
